-- -------------------------------------------------------------------------- --
--               TRASHERNET - A Trashy Ethernet Stack for FPGAs               --
-- -------------------------------------------------------------------------- --
-- trashernet_types.vhd : VHDL types used throughout Trashernet
-- -------------------------------------------------------------------------- --
-- Author       : Markus Koch <markus@notsyncing.net>
-- Contributors : None
-- License      : Mozilla Public License (MPL) Version 2
-- -------------------------------------------------------------------------- --

library ieee;
use ieee.std_logic_1164.all;

package trashernet_pkg is
	-- General types
	subtype byte is std_logic_vector(7 downto 0);
	type byte_vector is array (natural range <>) of byte;

	-- MAC specific types
	subtype mac_addr_t is byte_vector(0 to 5);
	subtype ethertype_t is byte_vector(0 to 1);

	type mac_header_fields is record
		mac_destination : mac_addr_t;   -- Destination MAC address
		mac_source      : mac_addr_t;   -- Source MAC address
		mac_ethertype   : ethertype_t;  -- Ethertype or length
	end record mac_header_fields;

	-- IP specific types
	subtype ip_addr_t is byte_vector(0 to 3);

	-- PHY interface
	type phy_in_t is record
		tx_data    : byte;              -- TX Data
		tx_data_en : std_logic;         -- Transmitter enable
	end record phy_in_t;
	type phy_out_t is record
		rx_data        : byte;          -- RX Data
		rx_data_valid  : std_logic;     -- RX Data valid
		rx_active      : std_logic;     -- RX of packet in progress

		tx_data_ack    : std_logic;     -- Latched data_tx
		tx_active      : std_logic;     -- Transmission in progress

		carrier_detect : std_logic;     -- Carrier detected
		rx_error       : std_logic;     -- Receive error
	end record phy_out_t;

	-- MAC interface
	type mac_in_t is record
		tx_header      : mac_header_fields; -- TX MAC Header Data
		tx_mac_data    : byte;          -- Payload
		tx_mac_data_en : std_logic;     -- Start (and keep) transmitting a frame
	end record mac_in_t;
	type mac_out_t is record
		rx_header         : mac_header_fields; -- RX MAC Header Data
		rx_mac_header_rcv : std_logic;  -- `rx_mac` header have been received and are valid

		rx_mac_data       : byte;       -- Ethernet data (after Ethertype)
		rx_mac_valid      : std_logic;  -- `rx_mac` values (headers + data) are valid
		tx_mac_data_ack   : std_logic;  -- The byte on `tx_mac_data` has been latched. Update to next word.
		tx_active         : std_logic;  -- Transmission in progress

		rx_mac_crc_ok     : std_logic;  -- End of packet, CRC OK (strobe independent from other rx_mac fields)
		rx_mac_crc_error  : std_logic;  -- End of packet, CRC invalid
	end record mac_out_t;

end package trashernet_pkg;

package body trashernet_pkg is

end package body trashernet_pkg;
